; ---------------------------------------------------------------
; Day Count Functions  -  by horst.schaeffer@gmx.net - for PB 4.0
; New version: 2006-02-08 - no global TD.SYSTEMTIME required
; ---------------------------------------------------------------
; The Day Count is a 32 bit count based on January 1, 1601 
; The same base is used for Windows' Filetime 
;   as a 64 bit 100 nanosecond count
; ---------------------------------------------------------------
; Functions: 
; DayCountFromDate(*td.SYSTEMTIME)        ; returns day count 
; DateFromDayCount(dayCount,*td.SYSTEMTIME) ; fills given SYSTEMTIME structure
; Today(DayCount,SYSTEMTIME)              ; macro  
; GetDateString(format$,DayCount)         ; formatted date from day count 
; GetDateFormat(type)                     ; get local format strings
; DateInputGadget(x,y,width,height,parent) ; date input w/ calendar 
; SetInputDate(handle,DayCount)           ; set date to input gadget
; GetInputDate(handle)                    ; get date count from input 


; .SYSTEMTIME to DayCount   -------------------------------------
; returns zero if invalid date 

#UnitsInSecond = 10000000
#UnitsInDay = 24*60*60*#UnitsInSecond

Procedure.l DayCountFromDate(*td.SYSTEMTIME)
  Protected ft.Q
  If SystemTimeToFileTime_(*td,@ft)
    ft / #UnitsInDay
  EndIf 
ProcedureReturn ft ; as long
EndProcedure 

; DayCount  to .SYSTEMTIME (result pointer given )---------------
; returns nonzero if successful

Procedure DateFromDayCount(dayCount,*td.SYSTEMTIME)
  Protected ft.Q
  ft.Q = dayCount
  ft * #UnitsInDay
  ProcedureReturn FileTimeToSystemTime_(@ft,*td)
EndProcedure 

; Day of Week (Sunday = 0, ... acc. to Win API) -----------------

Macro DOW(dayCount)
  (dayCount+1) % 7 
EndMacro 

; DayCount of Today ---------------------------------------------

; Procedure DCtoday()
;   Protected td.SYSTEMTIME
;   GetLocalTime_(@td)
; ProcedureReturn DayCountFromDate(@td) 
; EndProcedure 

; give: DayCount variable and pointer to SYSTEMTIME object
; example: Today(DC,@td.SYSTEMTIME)

Macro Today(DayCount,SYSTEMTIME)  ; not a function!
  GetLocalTime_(SYSTEMTIME)
  DayCount = DayCountFromDate(SYSTEMTIME) 
EndMacro  

; ---------------------------------------------------------------
; Make formatted date from DateCount or td.SYSTEMTIME 
; according to a given format string 
; The following tokens are suppoted in the format$ string (Windows API):
; (watch upper/lower case) - example: "yyyy-MM-dd"

; yyyy   yy    Year (4/2 digits) 
; MM     M     Month, numeric (with/without leading zero) 
; MMMM   MMM   Month by name (long/short) 
; dd     d     Day (with/without leading zero)
; dddd   ddd   Day of week (long/short)

; if given day_count = 0, current local system date is taken 

Procedure.s GetDateString(format$,DayCount)   
  Protected td.SYSTEMTIME, buf.s
  If DayCount : DateFromDayCount(DayCount,@td) : EndIf 
  buf.s = Space(64)
  GetDateFormat_(#LOCALE_USER_DEFAULT,0,@td,format$,buf,64)
ProcedureReturn buf
EndProcedure 

; ---------------------------------------------------------------
; Get LOCAL date format string with tokens & separators
; 2 types: 
#shortdate = #LOCALE_SSHORTDATE   ; e.g. 19.01.03
#longdate =  #LOCALE_SLONGDATE    ; e.g. Sonntag, 19. Januar 2003 

Procedure.s GetDateFormat(type)  ; type_ = #longdate or #shortdate
  Protected buf.s
  buf.s = Space(32)
  GetLocaleInfo_(#LOCALE_USER_DEFAULT,type,buf,32)
ProcedureReturn buf
EndProcedure 

; ----------------------------------------------------------------
  #MCSC_TITLEBK = 2
  #MCSC_TRAILINGTEXT = 5 
  
  #MCM_FIRST = $1000
  #MCM_GETCURSEL = #MCM_FIRST + 1 
  #MCM_SETCURSEL = #MCM_FIRST + 2
  #MCM_SETCOLOR = #MCM_FIRST + 10 
  #MCM_SETMONTHDELTA = #MCM_FIRST + 20
  
  #MCSC_BACKGROUND = 0
  #MCSC_TEXT = 1
  #MCSC_TITLEBK = 2 
  #MCSC_TITLETEXT = 3
  #MCSC_MONTHBK = 4
  #MCSC_TRAILINGTEXT = 5

  dt.INITCOMMONCONTROLSEX 
  dt\dwSize = SizeOf(INITCOMMONCONTROLSEX)
  dt\dwICC = #ICC_DATE_CLASSES
  InitCommonControlsEx_(@dt) 

; DateInputGadget() returns handle that is required for Get/SetDateInput()
; Standard local short date format 

Procedure DateInputGadget(x,y,width,height,parentHandle) 
  hCal=CreateWindowEx_(0,"SysDateTimePick32","DateTime",#WS_CHILD|#WS_VISIBLE,x,y,width,height,parentHandle,0,GetModuleHandle_(0),0) 
  SetWindowLong_(hCal,#GWL_STYLE,GetWindowLong_(hCal,#GWL_STYLE)|#WS_TABSTOP) 
ProcedureReturn hCal
EndProcedure 

  #DateInputEvent = 15  ; (? tested under Win98, W2K, XP) 

Procedure SetInputDate(handle,DayCount)
  Protected td.SYSTEMTIME 
  DateFromDayCount(DayCount,@td)
  SendMessage_(handle,#MCM_SETCURSEL,0,@td) 
EndProcedure 

Procedure GetInputDate(handle) 
  Protected td.SYSTEMTIME 
  SendMessage_(handle,#MCM_GETCURSEL,0,@td)
ProcedureReturn DayCountFromDate(@td)
EndProcedure 


; IDE Options = PureBasic 4.31 (Windows - x86)
; CursorPosition = 126
; FirstLine = 103
; Folding = --
; DisableDebugger
; CompileSourceDirectory